// clock.cpp

#include "stdafx.h"
#include "Stage.h"
#include "clock.h"
#include "math.h"

//////////////////////////////////////////////////////////////////////
// CHand

CHand::CHand(double l)
{
	CreateRod(0, 0, 0, 0, 0, 0.5, 1, 16);
	SetColor(1, 1, 0);
	C3dShape r;
	r.CreateRod(0, 0, 0.25, 0, l-3, 0.25, 0.20, 16);
	r.SetColor(0, 0, 1);
	AddChild(&r);
	C3dShape c;
	c.CreateCone(0, l-3, 0.25, 0.75, TRUE, 0, l, 0.25, 0,
		         FALSE, 16);
	c.SetColor(1, 1, 0);
	AddChild(&c);
}

//////////////////////////////////////////////////////////////////////
// CGear

CGear::CGear(double r, double t, int teeth)
{
	double twopi =  6.28318530718;
	double r1 = r - 0.3;
	double r2 = r + 0.3;
	int nFaceVert = teeth * 4;
	int nVert = nFaceVert * 2;
    D3DVECTOR* Vertices = new D3DVECTOR[nVert];
	D3DVECTOR* pv = Vertices;
	double da = twopi / (teeth * 4);
	double a = 0;
	for (int i = 0; i < teeth; i++) {
		pv->x = r1 * cos(a);
		pv->y = r1 * sin(a);
		pv->z = 0;
		pv++;
		a += da;
		pv->x = r2 * cos(a);
		pv->y = r2 * sin(a);
		pv->z = 0;
		pv++;
		a += da;
		pv->x = r2 * cos(a);
		pv->y = r2 * sin(a);
		pv->z = 0;
		pv++;
		a += da;
		pv->x = r1 * cos(a);
		pv->y = r1 * sin(a);
		pv->z = 0;
		pv++;
		a += da;
	}

	pv = Vertices;
	D3DVECTOR* pv2 = &Vertices[nFaceVert];
	for (i = 0; i < nFaceVert; i++) {
		*pv2 = *pv;
		pv2->z = t;
		pv++;
		pv2++;
	}

	// Generate face data for teeth.
	// Cross your fingers.
	int nf = (teeth * 5 * 4) + (teeth * 26) + 10;
    int* FaceData = new int[nf];
    int* pfd = FaceData;

	for (i = 0; i < teeth*4; i++) {
		*pfd++ = 4;
		*pfd++ = i;
		*pfd++ = (i + 1) % (teeth*4);
		*pfd++ = nFaceVert + ((i + 1) % (teeth*4));
		*pfd++ = nFaceVert + (i % (teeth*4));
	}

	// End the list
	*pfd++ = 0;

    Create(Vertices, nVert, NULL, 0, FaceData, TRUE);

	// Add end faces using normals 
	D3DVECTOR nvect [] = {
		{0, 0, 1},
		{0, 0, -1}
	};

   	delete [] FaceData;
	FaceData = new int [teeth * 9  + teeth * 4  + 10];

	pfd = FaceData;
	for (i = 0; i < teeth; i++) {
		*pfd++ = 4;
		*pfd++ = i*4;
		*pfd++ = 1;
		*pfd++ = i*4+3;
		*pfd++ = 1;
		*pfd++ = i*4+2;
		*pfd++ = 1;
		*pfd++ = i*4+1;
		*pfd++ = 1;
	}
	*pfd++ = teeth*2;
 	for (i = teeth-1; i >= 0; i--) {
		*pfd++ = i*4+3;
		*pfd++ = 1;
		*pfd++ = i*4;
		*pfd++ = 1;
	}
	*pfd++ = 0;
    AddFaces(Vertices, nVert, nvect, 2, FaceData);

	pfd = FaceData;
	for (i = 0; i < teeth; i++) {
		*pfd++ = 4;
		*pfd++ = nFaceVert + i*4;
		*pfd++ = 0;
		*pfd++ = nFaceVert + i*4+1;
		*pfd++ = 0;
		*pfd++ = nFaceVert + i*4+2;
		*pfd++ = 0;
		*pfd++ = nFaceVert + i*4+3;
		*pfd++ = 0;
	}
	*pfd++ = teeth*2;
 	for (i = 0; i < teeth; i++) {
		*pfd++ = nFaceVert + i*4;
		*pfd++ = 0;
		*pfd++ = nFaceVert + i*4+3;
		*pfd++ = 0;
	}
	*pfd = 0;
    AddFaces(Vertices, nVert, nvect, 2, FaceData);
	delete [] Vertices;
   	delete [] FaceData;

	SetColor(1, 1, 0);
}
